@@ -1,6 +1,8 @@
 /*
     SPDX-FileCopyrightText: 2008 Michael Jansen <kde@michael-jansen.biz>
     SPDX-FileCopyrightText: 2022 Ahmad Samir <a.samirh78@gmail.com>
+    SPDX-FileCopyrightText: 2026 Nicolas Fella <nicolas.fella@gmx.de>
+    SPDX-FileCopyrightText: 2026 Kristen McWilliam <kristen@kde.org>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
@@ -29,6 +31,8 @@
 #include <QPluginLoader>
 #include <QStandardPaths>
 
+using namespace Qt::StringLiterals;
+
 static bool checkPlatform(const QJsonObject &metadata, const QString &platformName)
 {
     const QJsonArray platforms = metadata.value(QStringLiteral("MetaData")).toObject().value(QStringLiteral("platforms")).toArray();
@@ -446,7 +450,7 @@ bool GlobalShortcutsRegistry::keyEvent(int keyQt, ShortcutKeyState state)
         if (state == ShortcutKeyState::Released) {
             if (m_state == PressingModifierOnly) {
                 m_state = ReleasingModifierOnly;
-                handled = processKey(m_currentModifiers, ShortcutKeyState::Pressed);
+                processKey(m_currentModifiers, ShortcutKeyState::Pressed);
             }
             m_currentModifiers = modifiers & ~Utils::keyToModifier(key);
             if (m_state == ReleasingModifierOnly && !m_currentModifiers) {
@@ -477,6 +481,17 @@ bool GlobalShortcutsRegistry::keyEvent(int keyQt, ShortcutKeyState state)
     return handled;
 }
 
+bool GlobalShortcutsRegistry::isShortcutAllowed(const GlobalShortcut *shortcut) const
+{
+    if (!m_useAllowList) {
+        return true;
+    }
+
+    return std::any_of(m_allowedShortcuts.cbegin(), m_allowedShortcuts.cend(), [shortcut](const ShortcutName &sc) {
+        return sc.componentName == shortcut->context()->component()->uniqueName() && sc.shortcutName == shortcut->uniqueName();
+    });
+}
+
 bool GlobalShortcutsRegistry::processKey(int keyQt, ShortcutKeyState state)
 {
     int keys[maxSequenceLength] = {0, 0, 0, 0};
@@ -546,6 +561,10 @@ bool GlobalShortcutsRegistry::processKey(int keyQt, ShortcutKeyState state)
         m_lastShortcut->context()->component()->emitGlobalShortcutEvent(*m_lastShortcut, ShortcutKeyState::Released);
     }
 
+    if (!isShortcutAllowed(shortcut)) {
+        return false;
+    }
+
     // Invoke the action
     shortcut->context()->component()->emitGlobalShortcutEvent(*shortcut, state);
     m_lastShortcut = shortcut;
@@ -627,6 +646,32 @@ KServiceActionComponent *GlobalShortcutsRegistry::createServiceActionComponent(c
     return static_cast<KServiceActionComponent *>(c);
 }
 
+void GlobalShortcutsRegistry::loadAllowListSettings()
+{
+    KConfig config(u"kglobalaccelrc"_s);
+    m_useAllowList = config.group(u"General"_s).readEntry("useAllowList", false);
+    m_allowedShortcuts.clear();
+
+    const KConfigGroup allowedGroup = config.group(u"AllowedShortcuts"_s);
+    const QStringList componentNames = allowedGroup.keyList();
+    for (const QString &componentName : componentNames) {
+        if (componentName.isEmpty()) {
+            qCWarning(KGLOBALACCELD) << "Skipping allow list entry with empty component name";
+            continue;
+        }
+
+        const QStringList shortcuts = allowedGroup.readEntry(componentName, QStringList());
+        for (const QString &shortcutName : shortcuts) {
+            if (shortcutName.isEmpty()) {
+                qCWarning(KGLOBALACCELD) << "Skipping allow list entry with empty shortcut name for component" << componentName;
+                continue;
+            }
+
+            m_allowedShortcuts << ShortcutName{componentName, shortcutName};
+        }
+    }
+}
+
 void GlobalShortcutsRegistry::loadSettings()
 {
     if (!m_components.empty()) {
@@ -725,6 +770,8 @@ void GlobalShortcutsRegistry::loadSettings()
     }
 
     detectAppsWithShortcuts();
+
+    loadAllowListSettings();
 }
 
 void GlobalShortcutsRegistry::detectAppsWithShortcuts()
